package com.atlassian.confluence.jspwiki;

import bucket.util.FileUtils;
import com.atlassian.confluence.core.ConfluenceEntityObject;
import com.atlassian.confluence.importexport.ImportExportException;
import com.atlassian.confluence.importexport.ImportExportManager;
import com.atlassian.confluence.importexport.impl.BackupExporter;
import com.atlassian.confluence.pages.Attachment;
import com.atlassian.confluence.pages.Page;
import com.atlassian.confluence.security.SpacePermission;
import com.atlassian.confluence.spaces.Space;
import com.atlassian.confluence.spaces.SpaceDescription;
import com.ecyrd.jspwiki.PageManager;
import com.ecyrd.jspwiki.WikiPage;
import com.ecyrd.jspwiki.attachment.AttachmentManager;
import com.ecyrd.jspwiki.providers.ProviderException;
import com.ecyrd.jspwiki.providers.WikiPageProvider;
import com.opensymphony.util.TextUtils;

import java.io.File;
import java.io.InputStream;
import java.sql.Timestamp;
import java.util.*;

public class JspWikiBackupExporter extends BackupExporter
{
    private ContentCleaner contentCleaner;
    private AttachmentManager attachmentManager;
    private PageManager pageManager;
    private long next = 1;
    private String spaceName;
    private String creatorUserName;
    private Space space;
    private Map pagesMap = new HashMap();
    private List attachments = new ArrayList();

    public JspWikiBackupExporter(PageManager pageManager, AttachmentManager attachmentManager, String spaceName, String creatorUserName, Properties properties)
    {
        this.pageManager = pageManager;
        this.attachmentManager = attachmentManager;
        this.spaceName = spaceName;
        this.creatorUserName = creatorUserName;

        Properties variables = new Properties(properties);
        variables.setProperty(ContentCleaner.VAR_JSPWIKI_VERSION, "2.0.52");//?
        variables.setProperty(ContentCleaner.VAR_PAGE_NAME, "This page");//haha good one!
        variables.setProperty(ContentCleaner.VAR_TOTAL_PAGES, String.valueOf(pageManager.getTotalPageCount()));
        this.contentCleaner = new ContentCleaner(variables);
    }

    protected String getExportType() throws ImportExportException
    {
        return ImportExportManager.EXPORT_XML_TYPE_SPACE;
    }

    protected List getObjectsForExport() throws ImportExportException
    {
        ArrayList objects = new ArrayList();
        addSpaceForExport(objects);
        addPagesForExport();
        addAttachmentsForExport();

        return objects;
    }

    private void addSpaceForExport(ArrayList objects)
    {
        space = new Space();
        space.setKey(spaceName);
        space.setName(spaceName);
        setCreationModificationInfo(space);
        SpaceDescription description = new SpaceDescription();
        setCreationModificationInfo(description);
        description.setContent(spaceName);
        space.setDescription(description);
        description.setSpace(space);

        // grant the creator all space permissions
        for (Iterator it = SpacePermission.GENERIC_SPACE_PERMISSIONS.iterator(); it.hasNext();)
        {
            String permissionType = (String) it.next();
            SpacePermission permission = new SpacePermission(permissionType, space, null, creatorUserName);
            space.addPermission(permission);
            setCreationModificationInfo(permission);
        }

        objects.add(space);
    }

    private void setCreationModificationInfo(ConfluenceEntityObject entity)
    {
        entity.setId(next++);
        entity.setCreationDate(new Timestamp(new Date().getTime()));
        entity.setCreatorName(creatorUserName);
        entity.setLastModificationDate(new Timestamp(new Date().getTime()));
        entity.setLastModifierName(creatorUserName);
    }

    private void addPagesForExport() throws ImportExportException
    {
        Collection allPages = null;

        try
        {
            allPages = pageManager.getAllPages();
        }
        catch (ProviderException e)
        {
            throw new ImportExportException(e);
        }

        for (Iterator allPagesIter = allPages.iterator(); allPagesIter.hasNext();)
        {
            WikiPage wikiPage = (WikiPage) allPagesIter.next();
            if (unExportableWikiPage(wikiPage))
            {
                continue;
            }

            try
            {
                Page page = createPage(wikiPage);
                List historicalVersions = createHistoricalVersions(wikiPage);

                if (!historicalVersions.isEmpty())
                {
                    for (Iterator iterator = historicalVersions.iterator(); iterator.hasNext();)
                    {
                        Page historicalConfluencePage = (Page) iterator.next();
                        page.addPreviousVersion(historicalConfluencePage);
                    }
                }

                space.addPage(page);
                pagesMap.put(page.getTitle(), page);
            }
            catch (Exception e)
            {
                e.printStackTrace();
                System.out.println("exporting page failed: " + wikiPage.getName());
            }
        }
    }

    private List createHistoricalVersions(WikiPage latestVersion) throws Exception
    {
        List result = new ArrayList();

        List versionHistory = pageManager.getVersionHistory(latestVersion.getName()); // jspwiki returns all versions of the page back (including the latest)
        if (versionHistory.size() > 1)
        {
            for (Iterator iterator = versionHistory.iterator(); iterator.hasNext();)
            {
                WikiPage historicalWikiPage = (WikiPage) iterator.next();

                if (historicalWikiPage.getVersion() != latestVersion.getVersion()) // if not the latest version
                    result.add(createPage(historicalWikiPage));
            }
        }
        return result;
    }

    private void addAttachmentsForExport() throws ImportExportException
    {
        Collection allAttachments = null;

        try
        {
            allAttachments = attachmentManager.getAllAttachments();
        }
        catch (ProviderException e)
        {
            throw new ImportExportException(e);
        }

        for (Iterator allPagesIter = allAttachments.iterator(); allPagesIter.hasNext();)
        {
            com.ecyrd.jspwiki.attachment.Attachment wikiAttachment = (com.ecyrd.jspwiki.attachment.Attachment) allPagesIter.next();

            try
            {
                Attachment attachment = createAttachment(wikiAttachment);
                Page page = (Page) pagesMap.get(wikiAttachment.getParentName());
                if (page != null)
                {
                    page.addAttachment(attachment);
                    attachments.add(attachment);

                    InputStream in = attachmentManager.getAttachmentStream(wikiAttachment);
                    FileUtils.copyFile(in, new File(new File(getWorkingExportContext().getExportDirectory() + File.separator + "attachments", page.getIdAsString()), attachment.getFileName()));
                }
            }
            catch (Exception e)
            {
                e.printStackTrace();
                System.out.println("exporting attachment failed: " + wikiAttachment.getFileName());
            }
        }
    }

    private Attachment createAttachment(com.ecyrd.jspwiki.attachment.Attachment wikiAttachment)
    {
        Attachment attachment = new Attachment();
        setCreationModificationInfo(attachment);
        attachment.setFileName(wikiAttachment.getFileName());
        attachment.setFileSize(wikiAttachment.getSize());
        attachment.setContentType("application/unknown");

        return attachment;
    }

    private Page createPage(WikiPage wikiPage) throws Exception
    {
        Page page = new Page();
        setCreationModificationInfo(page);
        page.setTitle(wikiPage.getName());
        String pageText = pageManager.getPageText(wikiPage.getName(), wikiPage.getVersion());
        page.setContent(clean(pageText));
        page.setVersion(wikiPage.getVersion());
        page.setLastModifierName(getAuthor(wikiPage));

        // creator is the author of the version 1
        WikiPage firstVersion = pageManager.getPageInfo(wikiPage.getName(), 1);
        page.setCreatorName(getAuthor(firstVersion));

        return page;
    }

    private String getAuthor(WikiPage wikiPage)
    {
        String author = wikiPage.getAuthor();
        if (TextUtils.stringSet(author))
            return author;
        else
            return creatorUserName;
    }

    private String clean(String pageText)
    {
        return contentCleaner.clean(pageText);
    }

    private boolean unExportableWikiPage(WikiPage wikiPage)
    {
        String name = wikiPage.getName();
        return name.equalsIgnoreCase("LeftMenu");
    }

    protected List getSourceAttachmentsDirForCopying()
    {
        //we already copied all files to attachments folder of the export! Just return an empty list
        return new ArrayList();
    }
}
